// FILE: approval_components/js/controllers/approvalHeaderController.js
/**
 * The controller of header template
 * @param {TemplateManager} templateManager - An instance of TemplateManager
 */
Configuration(function (eventEngine) {
	eventEngine.registerEvent("WidgetOnBack", "widgetOnBack");
});

function approvalHeaderController($scope, templateManager, ScreenService, ApplicationContext) {

	var self = this;

	var showAsActionTypes = {
		always: "always",
		list: "list",
		view: "view",
		edit: "edit",
		checked_rows: "checked_rows",
		never: "never",
		never_list: "never_list",
		never_view: "never_view",
		never_edit: "never_edit",
		never_checked_rows: "never_checked_rows",
		back: "back",
		back_list: "back_list",
		back_view: "back_view",
		back_edit: "back_edit",
		back_checked_rows: "back_checked_rows"
	};
	var searchStatus = {
		opened: "SO",
		closed: "SC",
		notSearching: "NS"
	};


	/**
	 * Retrieves actions conllection length
	 * @param  {String} type - the action type
	 * @return {Integer} Returns the length of actions collection by type
	 */
	this.getActionsLengthByType = function (type) {
		return getActionsByType(type).length;
	};

	/**
	 * Retrieves actions by type
	 * @param  {String} type - the action type
	 * @return {Array} Array of actions by type
	 */
	this.getActionsByType = getActionsByType;
	function getActionsByType(type) {
		return getActions().filter(function (action) {
			return findShowAsAction(type, action) && action.isVisible;
		});
	}

	function findShowAsAction(types, action) {
		var exist = false;
		types = types.split('|');
		var asAction = action.showAsAction.split('|');
		for (var i = asAction.length - 1; i >= 0 && exist === false; i--) {
			exist = showAsActionTypes[types[0]] == asAction[i] ? true : showAsActionTypes[types[1]] == asAction[i] ? true : false;
		}
		return exist;
	}

	this.getActionsVisibleLength = function (nameArrayActions) {
		return $scope[nameArrayActions].filter(function (action) {
			return action.isVisible;
		}).length;
	};

	/**
	 * Toggle dropdown visibility with fast animation
	 */
	this.toggleHeaderDropdown = function () {
		$scope.$root.headerDropdownOpened = !$scope.$root.headerDropdownOpened;
	};
	window.toggleHeaderDropdown = this.toggleHeaderDropdown;

	/**
	 * Retrieves the container's active widget
	 * @return {Widget} Active widget
	 */
	this.getActiveWidget = getActiveWidget;
	function getActiveWidget() {
		return getContainer().getActiveWidget();
	}

	/**
	 * Retrieves active widget's current row of current container
	 * @return {Object} Current row
	 */
	this.getCurrentRow = function () {
		return getActiveWidget().currentRow;
	};

	/**
	 * Execute selected action and toggle dropdown
	 * @param  {Action} action - selected action
	 * @param  {Function} actionClick - Zeedhi action click event
	 */
	this.selectDropdownOption = function (action, actionClick) {
		actionClick(action, getActiveWidget());
	};

	/**
	 * Attach actions and methods in container
	 */
	this.updateContainer = updateContainer;
	function updateContainer() {
		var container = getContainer();
		updateCurrentWidget(container);
		container.eachWidget(function (widget) {
			widget.getHeaderActions = getHeaderActions;
		});
	}


	function updateCurrentWidget(container) {
		var widget = getActiveWidget();
		widget.headerActions = widget.headerActions || [];
		widget.headerActions = factoryActions(container, widget.headerActions);
		widget.searchCriteria = widget.searchCriteria || {};
		updateSearching(widget);
		updateHeaderShadow(widget);
		updateHeaderForTWWI(widget, searchStatus.notSearching);
	}
	function updateHeaderForTWWI(widget, search) {
		var fatherTemp = widget.getParent().template;
		var srcTemp = 'approvaltabs#widget/tabbedWidgetWithImage.html';
		var ruiHeader = $('.approvalheader');
		var ruiContainer = $('.container[zh-fixed]');
		if (search === searchStatus.notSearching) {
			if (fatherTemp === srcTemp) {
				ruiHeader.addClass('header-over-tabs');
				ruiContainer.css('top', 0);
			} else {
				ruiHeader.removeClass('header-over-tabs');
				ruiHeader.removeClass('new-bg-header');
				ruiContainer.css('top', ruiHeader.css('height'));
			}
		} else if (search === searchStatus.opened && fatherTemp === srcTemp) {
			$('.approvalTWWI-Main-img').hide();
			$('.approvalheader').removeClass('header-over-tabs');
			$('.approvalheader').removeClass('new-bg-header');
			ruiContainer.css('top', ruiHeader.css('height'));
		}
		else if (search === searchStatus.closed && fatherTemp === srcTemp) {
			$('.approvalTWWI-Main-img').show();
			$('.approvalheader').addClass('header-over-tabs');
			$('.approvalheader').addClass('new-bg-header');
			ruiContainer.css('top', 0);
		}
	}
	function updateHeaderShadow(widget) {
		if (widget.parent && isTabbedTemplate(widget.parent.template)) {
			$('.approvalheader').removeClass('searchShadow');
		} else {
			$('.approvalheader').addClass('searchShadow');
		}
	}

	function isTabbedTemplate(template) {
		return template.search('tabbedWidget.html') !== -1 || template.search('tabbedWidgetWithImage.html') !== -1;
	}

	function updateSearching(widget) {
		if (widget.showSearch && widget.headerSearchBy) {
			$scope.searchVal = widget.searchCriteria._ALL;
		}

		if (widget.keepHeaderSearchingMode && searchFilled()) {
			$scope.$root.searching = true;
		} else if (searchFilled() && widget.mode == "list") {
			$scope.$root.searching = true;
		} else {
			$scope.$root.searching = false;
		}
	}
	this.getNeverMode = getNeverMode;
	function getNeverMode() {
		var widget = getActiveWidget();
		if (widget.hasCheckedRows()) {
			return "never_checked_rows";
		} else {
			if (widget.mode === "edit") {
				return "never_edit|never";
			} else if (widget.mode === "view") {
				return "never_view|never";
			} else if (widget.mode === "list") {
				return "never_list|never";
			}
		}
	}
	this.getBackMode = getBackMode;
	function getBackMode() {
		var widget = getActiveWidget();
		if (widget.hasCheckedRows()) {
			return "back_checked_rows";
		} else {
			if (widget.mode === "edit") {
				return "back_edit|back";
			} else if (widget.mode === "view") {
				return "back_view|back";
			} else if (widget.mode === "list") {
				return "back_list|back";
			}
		}
	}
	this.getCenterMode = getCenterMode;
	function getCenterMode() {
		var widget = getActiveWidget();
		if (widget.hasCheckedRows()) {
			return "checked_rows";
		} else {
			if (widget.mode === "edit") {
				return "edit|always";
			} else if (widget.mode === "view") {
				return "view|always";
			} else if (widget.mode === "list") {
				return "list|always";
			}
		}
	}

	/**
	 * Build actions applying mobile and default properties
	 * @param  {Container} container - current container
	 * @param  {Array} actions - array of actions to build
	 * @return {Array} An array of builded actions
	 */
	function factoryActions(container, actions) {
		return actions.map(function (action) {
			action = Util.extend(new Action(), action);
			_extendMobileProperties(action);
			action.container = container;
			action.widget = getActiveWidget();
			return action;
		});
	}

	/**
	 * Set mobile properties if it's a mobile device
	 * @param  {Object} obj - action object
	 */
	function _extendMobileProperties(obj) {
		if (Util.isMobile() && obj.mobile) {
			var mobile = obj.mobile;
			Object.keys(mobile).forEach(function extendMobileProperty(propertyToExtend) {
				if (obj.replaceByMobile || !obj[propertyToExtend] || typeof obj[propertyToExtend] !== 'object') {
					obj[propertyToExtend] = mobile[propertyToExtend];
				} else {
					obj[propertyToExtend] = Util.extend(obj[propertyToExtend], mobile[propertyToExtend]);
				}
			});
		}
	}

	/**
	 * Retrieves header actions of current container
	 * @return {Array} An array of actions
	 */
	function getActions() {
		return getActiveWidget().headerActions || [];
	}

	/**
	 * Retrieves current container
	 * @return {Container} Current container
	 */
	function getContainer() {
		return templateManager.container;
	}

	function widgetOnChangeMode(args) {
		if (getContainer()) {
			var widget = getActiveWidget();
			if (widget.mode != "list") {
				$scope.$root.searching = false;
			}
			updateContainer(getContainer());
		}
	}

	$scope.$watch(function () {
		return getContainer() ? getActiveWidget().mode : null;
	}, widgetOnChangeMode);

	$scope.$watch(function () {
		return getContainer() ? getActiveWidget().hasCheckedRows() : null;
	}, function (args) {
		if (getContainer())
			updateCurrentWidget(getContainer());
	});

	$scope.$watch(function () {
		return getContainer() ? getActiveWidget() : null;
	}, function (args) {
		if (getContainer())
			updateCurrentWidget(getContainer());
	});

	$scope.$watch(function () {
		return getContainer();
	}, function (args) {
		if (getContainer()) {
			updateContainer();
		}
	});

	/**
	 * Change header layout to search mode
	 * @param  {Boolean} type - true: Search mode, false: Normal mode
	 */
	this.toggleHeaderSearch = function (type) {
		var widget = getActiveWidget();
		if (type) {
			$scope.$root.searching = true;
			$('.approvaltabsMobileCont').css('top', '-50px');
			$('.approvalTWWI-mainContainer').css('top', '-50px');
			setTimeout(function () {
				$('#search-input').attr("placeholder", ScreenService.i18n("Search..."));
				$('#search-input').focus();
				updateHeaderForTWWI(widget, searchStatus.opened);
			}, 150);
		} else {
			if (searchFilled()) {
				clearSearch();
			}
			$('.approvaltabsMobileCont').css('top', '0');
			$('.approvalTWWI-mainContainer').css('top', '0');
			$('#search-input').attr("placeholder", "");
			$scope.$root.searching = false;
			updateHeaderForTWWI(widget, searchStatus.closed);
		}
	};
	window.toggleHeaderSearch = this.toggleHeaderSearch;

	/**
	 * Update the searchCriteria of active widget
	 */
	this.searchInputChange = searchInputChange;
	function searchInputChange() {
		var inputVal = $scope.searchVal;
		var widget = getActiveWidget();
		var showSearch = widget.showSearch;
		if (showSearch && !widget.headerSearchBy) {
			console.error("ERROR: headerSearchBy property not defined on widget '" + widget.name + "'.");
		} else {
			widget.headerSearchBy = Array.isArray(widget.headerSearchBy) ? widget.headerSearchBy : [widget.headerSearchBy];
			if (!Util.isEmptyOrBlank(inputVal)) {
				widget.searchCriteria._ALL = inputVal;
			} else {
				delete widget.searchCriteria._ALL;
			}

			if ($scope.searchTimeout) {
				clearTimeout($scope.searchTimeout);
			}
			$scope.searchTimeout = setTimeout(setSearchFilter.bind(this), 500);
		}
	}

	/**
	 * Set dataSourceFilter of widget according to searchCriteria
	 */
	function setSearchFilter() {
		var widget = getActiveWidget();

		if (!Object.keys(widget.searchCriteria).length) {
			widget.dataSourceFilter = widget.defaultDataSourceFilter || widget.dataSourceFilter || [];
		} else {
			if (!widget.defaultDataSourceFilter) {
				widget.defaultDataSourceFilter = widget.dataSourceFilter || [];
			}
			var name = widget.headerSearchBy.join("|");
			var operator = "LIKE_ALL";
			var value = '%' + widget.searchCriteria._ALL + '%';
			addFilterCondition(widget, name, operator, value);
		}
		widget.reload();
	}

	/**
	 * Add condition to widget's dataSourceFilter
	 * @param {Widget} widget
	 * @param {String} name
	 * @param {String} operator
	 * @param {String} value
	 */
	function addFilterCondition(widget, name, operator, value) {
		var dataSourceFilter = widget.dataSourceFilter || [];
		dataSourceFilter = dataSourceFilter.filter(function (row) {
			return row.name != name;
		});
		// if (!Util.isUndefinedOrNull(inDataSourceFilter[0])) {
		// 	dataSourceFilter.splice(inDataSourceFilter[0], 1);
		// }
		dataSourceFilter.push({
			"name": name,
			"operator": operator,
			"value": value
		});
		widget.dataSourceFilter = dataSourceFilter;
	}

	/**
	 * Manage visibility of clear search icon
	 * @return {Boolean} Status of widget's searchCriteria property
	 */
	this.searchFilled = searchFilled;
	function searchFilled() {
		var widget = getActiveWidget();
		if (widget.searchCriteria && widget.headerSearchBy) {
			return !Util.isEmptyOrBlank(widget.searchCriteria._ALL);
		} else {
			return false;
		}
	}

	/**
	 * Clear value of search input
	 */
	this.clearSearch = clearSearch;
	function clearSearch() {
		$scope.searchVal = "";
		searchInputChange();
	}

	function getHeaderActions(name) {
		return this.headerActions.filter(function (item) {
			return item.name === name;
		}).shift();
	}

	this.getTitle = function () {
		var container = templateManager.container;
		var widget = getActiveWidget();
		var headerTitle = widget.title || container.label;

		if (widget.showCheckedRowsCounterOnHeader && widget.getCheckedRows().length) {
			headerTitle = widget.getCheckedRows().length;
		}

		return headerTitle;
	};

	this.menuActionVisibility = function () {
		var container = templateManager.container;
		var widget = getActiveWidget();

		if (!container.showMenu || widget.getCheckedRows().length) {
			return false;
		} else {
			if (widget.showMenu === undefined) {
				return true;
			} else {
				return widget.showMenu;
			}
		}
	};

	var isGoingDown = true;
	var actualPosition = 0;
	var headerSize = 67;
	window.addEventListener("scroll", function (event) {
		var tabsHeaderClasses =  document.getElementsByClassName("approval-tab-header")[0];
		tabsHeaderClasses = tabsHeaderClasses ? tabsHeaderClasses.classList:null;
		
		var headerClasses = document.getElementsByTagName("header")[0].classList;
		var top = this.scrollY;

		isGoingDown = (actualPosition < top) && (top > headerSize);
		actualPosition = top;

		if (isGoingDown) {
			headerClasses.add("headerOut");
			if(tabsHeaderClasses){
				tabsHeaderClasses.add("fixAtTop");
			}
		} else if (top > headerSize && !isGoingDown) {
			headerClasses.remove("headerOut");
			if(tabsHeaderClasses){
				tabsHeaderClasses.remove("fixAtTop");
			}
		} else {
			headerClasses.remove("headerOut");
			if(tabsHeaderClasses){
				tabsHeaderClasses.remove("fixAtTop");
			}
		}
	}, false);

	$scope.showFilter = function () {
		filterBackup = Object.assign({},$scope.filterValues);
		$scope.filterDropdownOpened = true;
		document.body.classList.add('no-scroll');
		let awaitRenderFilter = setInterval(function(){
			const jQueryEl = $('.filter-body');
			if(jQueryEl && jQueryEl[0]){
				jQueryEl.hide().show(0);
				clearInterval(awaitRenderFilter);
			}
		}, 100);
	};
	
	$scope.hideFilter = function () {
		$scope.filterDropdownOpened = false;
		document.body.classList.remove('no-scroll');
	};
	
	$scope.showOrderDropdown = function () {
		$scope.orderOptions = templateManager.container.getActiveWidget().orderOptions;
		$scope.orderDropdownOpened = true;
		document.body.classList.add('no-scroll');
	};
	
	$scope.hideOrderDropdown = function () {
		$scope.orderDropdownOpened = false;
		document.body.classList.remove('no-scroll');
	};
	
	$scope.changeOrdenation = function(value, label) {
		let widget = templateManager.container.getActiveWidget();
		let orderByFilter = widget.dataSourceFilter.find(f => f.operator === 'ORDER_BY');
		if($scope.ordenation.name === value) {
			$scope.ordenation.direction = $scope.ordenation.direction === "ASC" ? "DESC" : "ASC";
			orderByFilter.name[0].direction = $scope.ordenation.direction; 
			$scope.selectedOrdernationClass = $scope.selectedOrdernationClass === "arrow-down" ? "arrow-up" : "arrow-down";
		} else {
			$scope.ordenation.name = value;
			orderByFilter.name[0].name = value;
		}
		orderByFilter.label = $scope.ordenation.label = label;
		widget.reload();
		scroll(0,0);
	};
	
	$scope.selectedOrdernationClass = "arrow-down";
	$scope.ordenation = {
    	name: "DTINCLUSAO",
    	direction: "DESC",
    	label: ScreenService.i18n("Data") 
	};
	
	$scope.childrenScopeInterface = {
		hide: $scope.hideFilter,
		show: $scope.showFilter,
	};
	
}

// FILE: approval_components/js/controllers/ApprovalTabsController.js
function ApprovalTabsController($scope, $element) {
    // $scope.widgets = $scope.widget.widgets;
    
    let pageSize = window.innerWidth;
    let tabsContainer;
    $scope.selectedPage = 0;

    $scope.onInit = function(argument) {
        let rootElement = $element.get(0);
        tabsContainer = rootElement.querySelector('.tab-content-container');
        // let tabHeader = rootElement.querySelector('.approval-tab-header');
        // setTimeout(() => {
        //     rootElement.classList.add('slidein-animation');
        //     // tabHeader.classList.add('animation');    
        // }, 290);
        
    };

    
    $scope.changeTab = function(tabIndex) {
        $scope.selectedPage = tabIndex;
        tabsContainer.scroll({
            left: tabIndex * pageSize,
            behavior: "smooth"
        }); 
    };
    
    
}

// FILE: approval_components/js/controllers/CardListController.js
function CardListController($scope, $element, ScreenService, ApplicationContext, MessageService) {

    let htmlElement = $element.get(0);
    let list;
    let isLoading = false;

    let dataSet = [];
    let widget;
    let dataSource;

    let fieldsToDisplayOnCard = [{
        "id": "NMALCADA",
        "label": "Alçada" 
    }, {
        "id": "VRTOTALAF",
        "label": "R$"
    }, {
        "id": "NRDEMANDA",
        "label": "Nº Demanda"
    }, {
        "id": "DTINCLUSAO",
        "label": "Incluído em:"
    }, {
        "id": "NMFILIAL",
        "label": "Unidade" 
    }, {
        "id": "IDCLASSIFIC",
        "label": "Classificação" 
    }, {
        "id": "NMOPERADOR",
        "label": "Solicitante:"
    }, {
        "id": "DSMOTVAFALCADA",
        "label": "Motivo do Lançamento:"
    }];


    /**
        called on ng-init
    */
    $scope.onAttach = function () {
        list = htmlElement.querySelector('.card-list-container');
        searchInput = htmlElement.querySelector('.search');
        widget = $scope.widget;
        dataSource = widget.dataSource;
        dataSet = widget.dataSource.data || [];
        defineDataProperty();
        if (widget.dataSource.data && widget.dataSource.data.length > 0) {
            renderList();
        }
        if (widget.enableInfiniteScroll !== false) {
            list.addEventListener('scroll', event => {
                if (!$scope.childrenVisiible && !isLoading && (list.offsetHeight + list.scrollTop >= list.scrollHeight)) {
                    isLoading = true;
                    dataSource.load(widget.dataSourceFilter, dataSource.getNextPage()).then(() => isLoading = false);
                }
            });
        }
        widget.cardListInitialized = true;
        // bindListeners();
    };

    function translate(fieldLabel) {
        return ScreenService.i18n(fieldLabel);
    }

    function removeCard(id, widget) {
        if (list) {
            for (var i = 0; i < list.children.length; i++) {
                if (list.children[i].key === id) {
                    list.removeChild(list.children[i]);
                    return;
                }
            }
        } else {
            var dataSource = widget.container.getWidget('alcadas').dataSource.data;
            index = dataSource.indexOf(dataSource.find(dts => dts.NRDEMANDA === id));
            dataSource = dataSource.splice(index, 1);
            widget.container.getWidget('alcadas').reload();
            return true;
        }
    }
    
    function approvesCompetency(isApproving, justification) {
        let key = widget.currentRow.NRDEMANDA;
        let operation = isApproving ? 'APROVAR' : 'REPROVAR';
        ApplicationContext.ApprovalService.approvesAlcada(widget.currentRow, operation, justification).then(responseData => {
            removeCard(key, widget);
            destroyJustificationBOX();
            let message = responseData.messages[0].message;
            if (message == 'emailNotSend') {
                MessageService.showMessageByCode("EMAIL_NOT_SEND");
            } else if(message == 'Aprovada com sucesso!') {
                MessageService.showMessageByCode("SUCCESS_APPROVED");
            } else if(message == 'Reprovada com sucesso!') {
                MessageService.showMessageByCode("SUCCESS_REJECT");
            }
            if (document.getElementsByClassName('details-action')[0]) {
                document.getElementsByClassName('details-action')[0].classList.add('hidden-actions');
            }
        }, errorResponse => {
            if(errorResponse.error === 'NO_ALCADA_WORKFLOW') {
                ApplicationContext.ApprovalService.getConsisteOrca(widget.currentRow).then(response => {
                    if(response && response.dataset && response.dataset.data && response.dataset.data.length) {
                        const data = response.dataset.data[0];

                        if(data.IDORIGEMCONS === 'CONSUMO') {
                            MessageService.showMessageByCode('NO_ALCADA_WORKFLOW_CONSUMO');
                        } else {
                            MessageService.showMessageByCode('NO_ALCADA_WORKFLOW_DESPESA', {CDDESPPREV: data.CDDESPPREV});
                        }                        
                    }
                });                
            } else {
                MessageService.getMessageByCode(errorResponse.error).then((message) => {
                    if(!isApproving) {
                        MessageService.showMessageByCode('FAIL_REJECT', {PARAM: message});
                    } else {
                        MessageService.showMessageByCode('FAIL_APPROVE', {PARAM: message});
                    }
                }, () => {
                    if(!isApproving) {
                        MessageService.showMessageByCode('FAIL_REJECT', {PARAM: errorResponse.error});
                    } else {
                        MessageService.showMessageByCode('FAIL_APPROVE', {PARAM: errorResponse.error});
                    }
                });
            }            
        });
    }

    function saveJustify(required) {
        let justification = document.getElementsByClassName('justification-box')[0].getElementsByTagName('textarea')[0].value;
        document.getElementsByClassName('justification-error')[0].classList.remove('show');
        if (!required || (required && justification.length > 10)) {
            widget.currentRow.justification = justification;
            approvesCompetency(!required, justification);
        } else {
            document.getElementsByClassName('justification-error')[0].classList.add('show');
            return false;
        }
        document.getElementsByTagName('body')[0].classList.remove('stop-scroll');
    }

    function cancelJustify(required) {
        document.getElementsByTagName('body')[0].classList.remove('stop-scroll');
        destroyJustificationBOX();
    }

    function destroyJustificationBOX() {
        let justificationBox = document.getElementsByClassName('justification-box')[0];
        justificationBox.classList.remove('open');
        setTimeout(function () {
            justificationBox = justificationBox.parentElement;
            justificationBox.remove(justificationBox);
        }, 400);
    }

    function renderJustificationBOX(required) {
        let overlay = document.createElement('div');
        overlay.classList.add('justification-box-overlay');

        let justificationBox = document.createElement('div');
        justificationBox.classList.add('justification-box');

        let textJustificationLabel = document.createElement('label');
        let requiredMark = !required ? "*" : "";

        let textJustification = document.createElement('textarea');
        let justificationError = document.createElement('span');
        justificationError.classList.add('justification-error');

        let actionsArea = document.createElement('div');
        actionsArea.classList.add('actions-area');

        let saveButton = document.createElement('button');
        

        let cancelButton = document.createElement('button');
        
        MessageService.getMessageByCode('JUSTIFY').then(function(message) {
            textJustificationLabel.textContent = message + requiredMark + ':';
        });
        
        MessageService.getMessageByCode('JUSTIFY_LENGTH').then(function(message) {
            justificationError.textContent = message;
        });
        
        MessageService.getMessageByCode('SAVE').then(function(message) {
            saveButton.textContent = message;
            saveButton.addEventListener('click', function () {
                saveJustify(required);
            }, false);
        });
        
        MessageService.getMessageByCode('CANCEL').then(function(message) {
            cancelButton.textContent = message;
            cancelButton.addEventListener('click', function () {
                cancelJustify();
            }, false);
        });

        actionsArea.appendChild(saveButton);
        actionsArea.appendChild(cancelButton);

        justificationBox.appendChild(textJustificationLabel);
        justificationBox.appendChild(textJustification);
        justificationBox.appendChild(justificationError);
        justificationBox.appendChild(actionsArea);

        overlay.append(justificationBox);

        return overlay;
    }

    function showJustificationBox(required) {

        let footer = document.getElementsByClassName('footer-container')[0].parentNode;
        footer.append(renderJustificationBOX(required));

        setTimeout(function () {
            let justificationBox = document.getElementsByClassName('justification-box')[0];
            justificationBox.classList.add('open');
        }, 100);
    }

    function startProcess(curAlcada, isApproving) {
        widget = widget ? widget : $scope.widget;
        widget.currentRow = widget.currentRow || widget.currentRow;
        widget.currentRow = Object.assign(widget.currentRow, curAlcada);
        widget.currentRow.isApproving = isApproving;

        let status = isApproving ? 'REALLY_APPROVE' : 'REALLY_REJECT';
        let required = isApproving ? false : true;
    
        MessageService.confirmMessageByCode(status, null, {
            type: 'question',
            success: () => {
                if(!isApproving) {
                    showJustificationBox(required);
                } else {
                    if(curAlcada.IDEXIGJUSTIFICATIVA === 'S') {
                        showJustificationBox(required);
                    } else {
                        approvesCompetency(isApproving, null);
                    }
                }
            },
            function() {
                let cardListContent = document.getElementsByTagName('body')[0];
                cardListContent.classList.add('stop-scroll');
                cardListContent.classList.remove('stop-scroll'); 
            }
        });



    }
    // $scope.onAttach = function() {
    $scope.startProcessByDetais = function (curAlcada, isApproving, widget) {
        isApproving = isApproving === "approved" ? true : false;
        startProcess(curAlcada, isApproving);
    };

    function startApprovedProcess(curDataRow) {
        startProcess(curDataRow, true);
    }

    function startRepprovedProcess(curDataRow) {
        startProcess(curDataRow, false);
    }

    function createChildElement(curDataRow) {

        let newCard = document.createElement('div');

        newCard.key = curDataRow.NRDEMANDA;
        newCard.classList.add('card-content');
        
        let firtsSection = null;
        let secondSection = null;
        let lastSection = null;
        
        for (var i = 0; i < fieldsToDisplayOnCard.length; i++) {
            let fieldTextContent = curDataRow[fieldsToDisplayOnCard[i].id] ? curDataRow[fieldsToDisplayOnCard[i].id] : '';
            let fieldLabel = "";
            if(fieldsToDisplayOnCard[i].id === "NMALCADA" || fieldsToDisplayOnCard[i].id === "NMFILIAL" || fieldsToDisplayOnCard[i].id === "IDCLASSIFIC")
                fieldLabel = ScreenService.i18n(fieldsToDisplayOnCard[i].label) + ":";
            else 
                fieldLabel = ScreenService.i18n(fieldsToDisplayOnCard[i].label)
            if (i === 0 && !firtsSection) {
                firtsSection = document.createElement('div');
                firtsSection.classList.add('firtsSection');
                newCard.appendChild(firtsSection);
            } else if (i === 2 && !secondSection) {
                secondSection = document.createElement('div');
                secondSection.classList.add('secondSection');
                newCard.appendChild(secondSection);
            } else if (i === 5 && !lastSection) {
                lastSection = document.createElement('div');
                lastSection.classList.add('lastSection');
                newCard.appendChild(lastSection);
            }

            let currentSection;
            if (i >= 0 && i <= 1) {
                currentSection = firtsSection; 
            } else if (i >= 2 && i <= 4) {
                currentSection = secondSection;
            } else if (i >= 5) {
                currentSection = lastSection;
            }

            let cardProperty = document.createElement('span');
            cardProperty.innerHTML = '<b>' + ScreenService.i18n(fieldLabel) + ' ' + '</b>' + fieldTextContent;
            currentSection.appendChild(cardProperty);
        }

        let actionsDiv = document.createElement('div');


        let fieldList = widget.fields;
        for (var i = 0; i < fieldList.length; i++) {
            var curAction = fieldList[i];
            if (!!curAction.isVisible) {
                let actionButton = document.createElement('button');
                if (curAction.isApproving === "approved") {
                    actionButton.addEventListener('click', function (event) {
                        event.stopPropagation();
                        startApprovedProcess(curDataRow);
                    }, false);
                } else {
                    actionButton.addEventListener('click', function (event) {
                        event.stopPropagation();
                        startRepprovedProcess(curDataRow);
                    }, false);
                }

                let actionButtonIcon = document.createElement('i');
                actionButtonIcon.classList.add('fas');
                actionButtonIcon.classList.add('fa-' + curAction.label);

                actionButton.appendChild(actionButtonIcon);
                actionsDiv.appendChild(actionButton);
            }
        }

        if (actionsDiv.childNodes.length > 0) {
            actionsDiv.classList.add('actions-container');
        }


        newCard.appendChild(actionsDiv);
        let viewDetailsAction = document.createElement('i');
        viewDetailsAction.className = 'fas fa-ellipsis-h card-details-action';
        viewDetailsAction.addEventListener('click', openChildren);

        let explandDetailsAction = document.createElement('i');
        explandDetailsAction.className = 'fas fa-chevron-down card-details-expander';
        explandDetailsAction.addEventListener('click', expandCard);

        newCard.appendChild(viewDetailsAction);
        newCard.appendChild(explandDetailsAction);
        return newCard;
    }

    let zhLoader = document.querySelector('div[data-zh-loader]');

    let expandCard = event => {
        let classList = event.target.parentElement.classList;
        if(classList.value.indexOf('expand-card') > 0){
            classList.remove('expand-card');
        }else{
            classList.add('expand-card');
        }
        
    };

    function setDataSourceFilter (detailsWidget, currentRow){
        detailsWidget.dataSourceFilter = [];
        let filter = ['CDFILLANC','NRDEMANDA','CDFILDEST'];
        filter.forEach(f => {
            detailsWidget.dataSourceFilter.push({
                "name": f,
                "operator": "=",
                "value": currentRow[f]
            });
        });
        detailsWidget.reload();
    }

    let openChildren = event => {
        let elementData = dataSet.find(dataElement => dataElement.NRDEMANDA === event.currentTarget.parentElement.key);
        if (widget.widgets && widget.widgets.length > 0) {
            zhLoader.classList.add('force-display-none');
            widget.currentRow = Object.assign({}, elementData);
            let detailsWidget = widget.widgets.find(w=> w.name === 'details').widgets.find(w=> w.name === 'description');
            if(detailsWidget){
                setDataSourceFilter(detailsWidget, widget.currentRow);
                detailsWidget.getField('approvalChart').isVisible = widget.currentRow.IDTPALCADA !== 'CATALOGO'; 
            }
            $scope.childrenVisiible = true;
            $scope.$root.activityStack = $scope.$root.activityStack || [];
            $scope.$root.activityStack.push(() => {
                $scope.childrenVisiible = false;
                widget.activate();
            });
            widget.widgets[0].activate();
            $scope.$apply();
        }
    };

    let openChildrenFromNotification = (event) => {
        console.log(event);
        let elementData = event.detail;
        let widget = $scope.widget;
        if (widget.widgets && widget.widgets.length > 0) {
            zhLoader.classList.add('force-display-none');
            widget.currentRow = Object.assign({}, elementData);
            let detailsWidget = widget.widgets.find(w=> w.name === 'details').widgets.find(w=> w.name === 'description');
            if(detailsWidget){
                setDataSourceFilter(detailsWidget, widget.currentRow);
            }
            $scope.childrenVisiible = true;
            $scope.$root.activityStack = $scope.$root.activityStack || [];
            $scope.$root.activityStack.push(() => {
                $scope.childrenVisiible = false;
                widget.activate();
            });
            widget.widgets[0].activate();
            // $scope.$apply();
        }
    };

    /**
     * @param {array} elementsToConcat an array of objects to render, if this param is falsy the entire dataSet will be rendered.
     */
    function renderList(elementsToConcat) {
        if (list) {
            if (!elementsToConcat) {
                list.innerHTML = '';
            }
            let elementsToRender = elementsToConcat || dataSet;

            if (dataSet.length === 0) {
                let emptyListWarning = document.createElement('span');
                MessageService.getMessageByCode('NO_DATA').then(function(message) {
                    emptyListWarning.innerHTML = message;
                    emptyListWarning.classList.add('empty-list-warning');
                    list.appendChild(emptyListWarning);
                });
            } else {
                elementsToRender.forEach(data => {
                    list.appendChild(createChildElement(data));
                });
            }
        } else {
            console.log('%cPANIC: an attempt to render elements before initial attach was made', 'font-size: 15px;background-color:red;');
        }
    }


    function defineDataProperty() {
        Object.defineProperty(dataSource, "data", {
            set: newValue => {
                if (dataSource.getCurrentPage() === 1) {
                    // render all values
                    dataSet = newValue;
                    renderList();
                } else {
                    // render only new values
                    let newValuesBegining = dataSet.length;
                    renderList(newValue.slice(newValuesBegining));
                    dataSet = newValue;
                }
            },
            get: () => dataSet
        });
    }

    document.addEventListener('notificationopen', openChildrenFromNotification);
    
}

// FILE: approval_components/js/controllers/ChartsController.js
function ChartsController($scope, $element, ScreenService) {

    let htmlElement = $element.get(0);
    let widget = $scope.widget;

    $scope.inicializeVars = function (field) {
        let itens = field.itens;
        let total = itens.find(function(filterEl) {
            return filterEl.isTotal === true;
        });
        let start = itens.find(function(filterEl) {
            return filterEl.isStartValue === true;
        });
        let curAdition = itens.find(function(filterEl) {
            return filterEl.isCurrAdition === true;
        });
        
        start.valor = 0;
        total.valor = 0;
        curAdition.valor = 0;
    };
}

// FILE: approval_components/js/controllers/FilterController.future.js
function ApprovalFilterController($scope, templateManager, ApplicationContext) {

	$scope.filterValues = {};
	$scope.fieldValues = {};
	let filterBackup;
	let dataSourceFilterBackup;
	
	activeWidget.filterProperties.filterValues = {};
	activeWidget.filterProperties.fieldValues = {};
	activeWidget.filterProperties.filterBackup = {};
	activeWidget.filterProperties.dataSourceFilterBackup = {};


	$scope.childrenScopeInterface.createBackup = function(){
		const activeWidget = templateManager.container.getActiveWidget();
		const filterProperties = activeWidget.filterProperties;
		let dataSourceFilter = activeWidget.dataSourceFilter;
		dataSourceFilterBackup = dataSourceFilter.map(filter => Object.assign({}, filter));
		filterProperties.filterBackup = Object.assign({}, filterProperties.filterValues);
	};
	
	$scope.childrenScopeInterface.restoreBackup = function() {
		const activeWidget = templateManager.container.getActiveWidget();
		const filterProperties = activeWidget.filterProperties;
		activeWidget.dataSourceFilter = filterProperties.dataSourceFilterBackup;
		// attributing filterValues one by one to trigger setters
		for (const filterElementName in filterProperties.filterBackup) {
			filterProperties.filterValues[filterElementName] = filterProperties.filterBackup[filterElementName];
		}
	};
	
	// @Deprecated
	function buildDateFilter(dataSourceFilter, dateFieldValue) {
		if(dateFieldValue === "") {
			if(dataSourceFilter.DTINICIAL){
				delete dataSourceFilter.DTINICIAL;
			}
			if(dataSourceFilter.DTFINAL){
				delete dataSourceFilter.DTFINAL;
			}
			dataSourceFilter.TDATAS = "T";
		}
	}
	
	// @Deprecated
	function buildFilterOld(widget){
		let valuesToConcatInDataSourceFilter = Object.assign({}, $scope.filterValues);
		
		let dateFieldValue = valuesToConcatInDataSourceFilter.dateFieldValueField;
		delete valuesToConcatInDataSourceFilter.dateFieldValueField;
		buildDateFilter(widget.dataSourceFilter, dateFieldValue);
		
		// overlap existing properties	
		widget.dataSourceFilter.forEach(filterValue => {
			
			if(valuesToConcatInDataSourceFilter.hasOwnProperty(filterValue.name)) {
				
				filterValue.value = valuesToConcatInDataSourceFilter[filterValue.name];
				if(filterValue.value === "") {
					filterValue.value = "T";
				}
				
				delete valuesToConcatInDataSourceFilter[filterValue.name];
			} 
			
		});
		
		// concat properties that don't exist yet
		for (let filterNmae in valuesToConcatInDataSourceFilter) {
			let valueToConcat = valuesToConcatInDataSourceFilter[filterNmae];
			if(valueToConcat === "") {
				valueToConcat = 'T';	
			}
			widget.dataSourceFilter.push({
				name: filterNmae,	
				operator: "=",	
				value: valueToConcat,	
			});
		}
		
	}
	
	function buildFilter(widget){
		const filterProperties = templateManager.container.getActiveWidget().filterProperties;
		let valuesToConcatInDataSourceFilter = Object.assign({}, filterProperties.filterValues);
		let fields = templateManager.container.getActiveWidget().filterProperties.fields;
		fields.forEach(f => {
			if(f.skipAutoFilter) 
				delete valuesToConcatInDataSourceFilter[f.name];
		});
		
		for (let filterNmae in valuesToConcatInDataSourceFilter) {
			let valueToConcat = valuesToConcatInDataSourceFilter[filterNmae];
			if(valueToConcat === "") {
				valueToConcat = 'T';	
			}
			let filterToChange = widget.dataSourceFilter.find(filter => filter.name === filterNmae);
			if(filterToChange) {
				filterToChange.value = 	valueToConcat;
			} else {
				widget.dataSourceFilter.push({
					name: filterNmae,	
					operator: "=",	
					value: valueToConcat,	
				});
			}
		}
		
	}
	
	$scope.applyFilter = function() {
		let activeWidget = templateManager.container.getActiveWidget();
		buildFilter(activeWidget);
		return activeWidget.reload();
	};	
	
	// @Deprecated
	$scope.onValueSelected = function (fieldValue, fieldName) {
		let activeWidget = templateManager.container.getActiveWidget();
		if(!activeWidget.filterValue) {
			activeWidget.filterValue = {};
		}
		activeWidget.filterValue[fieldName] = fieldValue;
	};
	
	function evaluateExpression(expression, context, withoutAtChar) {
		if(withoutAtChar || expression.match(/\@(.*?)\@/g)) {
			expression = expression.replace(/\@/g, '');
			return $scope.$eval(expression, context);
		} else {
			return expression;
		}
	}
	
	let reload = function(){
		this.aditionalOptions = this.aditionalOptions || [];
		ApplicationContext.RestEngine.request(this.dataSource.name, {
			requestType: "FilterData",
			itemsPerPage: 500,
			page: 1,
		    filter: this.dataSourceFilter || []
		}).then(response => {
		        $scope.fieldValues[this.name] = this.aditionalOptions.concat(response.dataset.filial);
		});
	};
	
	$scope.initFields = function(){
		const activeWidget = templateManager.container.getActiveWidget();
		const filterProperties = activeWidget.filterProperties;
		let fields = activeWidget.filterProperties.fields;
		
		activeWidget.filterProperties.filterValues = {};
		activeWidget.filterProperties.fieldValues = {};
		activeWidget.filterProperties.filterFieldsBackup = {};
		activeWidget.filterProperties.dataSourceFilterBackup = {};

		for (let i = 0; i < fields.length; i++) {
			let defaultValue = fields[i].default;
			if(defaultValue) {
				defaultValue = evaluateExpression(defaultValue);
				filterProperties.filterValues[fields[i].name] = defaultValue;
			}
			if(fields[i].events) {
				let fieldOnChangeEvent = fields[i].events.find((e) => e.name === 'FieldOnChange');
				if(fieldOnChangeEvent && fieldOnChangeEvent.code) {
					fields[i].onchange = fieldOnChangeEvent.code;
				}
			}
			if(fields[i].type === undefined || fields[i].type === 'select'){
				if(fields[i].dataSource && fields[i].dataSource.rest) {
					fields[i].reload = reload.bind(fields[i]);
					fields[i].reload();
				} else {
					$scope.fieldValues[fields[i].name] = fields[i].dataSource.data;
				}
			} else if(fields[i].type === 'toggle') {
				fields[i].valueToTrue = fields[i].valueToTrue || true; 
				fields[i].valueTofalse = fields[i].valueTofalse || false; 
			}
		}
		
	}
	
	$scope.triggerOnChange = function(field) {
		if(field.onchange) {
			let activeWidget = templateManager.container.getActiveWidget();
			evaluateExpression(
				field.onchange, 
				Object.assign(ApplicationContext, {
					filterValues: activeWidget.filterProperties.filterValues,
					currentWidget: activeWidget
				}), 
				true
			);
		}
	};
	
	$scope.initTest = function(){
		getData();	
	};
	
	$scope.beforeSelectAutoCompleteInitialized = (autoCompleteScope) => {
		/**
		 * the select auto complete field definition is not inside a widget,
		 * so we need to factor a dataSource by our own.
		 * :/
		 */
		autoCompleteScope.field.dataSource = metaDataFactory.dataSourceFactory(
			autoCompleteScope.field.dataSource,
			autoCompleteScope.field.dataSourceFilter,
			autoCompleteScope.field
		);
	};

	$scope.onSelectAutoCompleteInitialized = (autoCompleteScope) => {
		const activeWidget = templateManager.container.getActiveWidget();
		const filterProperties = activeWidget.filterProperties;
		autoCompleteScope.__fieldRow = filterProperties.filterValues[autoCompleteScope.field.name];
        Object.defineProperty(filterProperties.filterValues, autoCompleteScope.field.name, {
            get: function() {
                return autoCompleteScope.__fieldRow;
            },
            set: function(newValue) {
                autoCompleteScope.__fieldRow = newValue;
				// autoCompleteScope.widget.editRow[autoCompleteScope.field.name] = newValue;
				let selectedElement = autoCompleteScope.field.dataSource.data.find(dataSourceElement => {
					return dataSourceElement[autoCompleteScope.field.name] === autoCompleteScope.__fieldRow;
				});
				if(selectedElement){
					autoCompleteScope.descriptionText = autoCompleteScope.getDescriptionString(selectedElement);
				}
            },
            enumerable: true,
            configurable: true
		});
		// compute initial descrption text
		// if(!autoCompleteScope.descriptionText && autoCompleteScope.__fieldRow){
		// 	let selectedElement = autoCompleteScope.field.dataSource.data.find(dataSourceElement => {
		// 		return dataSourceElement[autoCompleteScope.field.name] === autoCompleteScope.__fieldRow;
		// 	});
		// 	if(selectedElement){
		// 		autoCompleteScope.descriptionText = autoCompleteScope.getDescriptionString(selectedElement);
		// 	}
			
		// }
	};
}

// FILE: approval_components/js/controllers/FilterController.js
function ApprovalFilterController($scope, templateManager, ScreenService, ApplicationContext) {

	$scope.filterValues = {};
	$scope.fieldValues = {};
	let filterBackup;
	let dataSourceFilterBackup;
	
	$scope.childrenScopeInterface.createBackup = function(){
		let dataSourceFilter = templateManager.container.getActiveWidget().dataSourceFilter;
		dataSourceFilterBackup = dataSourceFilter.map(filter => Object.assign({}, filter));
		filterBackup = Object.assign({}, $scope.filterValues);
	};
	
	$scope.childrenScopeInterface.restoreBackup = function() {
		templateManager.container.getActiveWidget().dataSourceFilter = dataSourceFilterBackup;
		// attributing filterValues one by one to trigger setters
		for (const filterElementName in filterBackup) {
			$scope.filterValues[filterElementName] = filterBackup[filterElementName];
		}
	};
	
	// @Deprecated
	function buildDateFilter(dataSourceFilter, dateFieldValue) {
		if(dateFieldValue === "") {
			if(dataSourceFilter.DTINICIAL){
				delete dataSourceFilter.DTINICIAL;
			}
			if(dataSourceFilter.DTFINAL){
				delete dataSourceFilter.DTFINAL;
			}
			dataSourceFilter.TDATAS = "T";
		}
	}
	
	// @Deprecated
	function buildFilterOld(widget){
		let valuesToConcatInDataSourceFilter = Object.assign({}, $scope.filterValues);
		
		let dateFieldValue = valuesToConcatInDataSourceFilter.dateFieldValueField;
		delete valuesToConcatInDataSourceFilter.dateFieldValueField;
		buildDateFilter(widget.dataSourceFilter, dateFieldValue);
		
		// overlap existing properties	
		widget.dataSourceFilter.forEach(filterValue => {
			
			if(valuesToConcatInDataSourceFilter.hasOwnProperty(filterValue.name)) {
				
				filterValue.value = valuesToConcatInDataSourceFilter[filterValue.name];
				if(filterValue.value === "") {
					filterValue.value = "T";
				}
				
				delete valuesToConcatInDataSourceFilter[filterValue.name];
			} 
			
		});
		
		// concat properties that don't exist yet
		for (let filterNmae in valuesToConcatInDataSourceFilter) {
			let valueToConcat = valuesToConcatInDataSourceFilter[filterNmae];
			if(valueToConcat === "") {
				valueToConcat = 'T';	
			}
			widget.dataSourceFilter.push({
				name: filterNmae,	
				operator: "=",	
				value: valueToConcat,	
			});
		}
		
	}
	function buildFilter(widget){
		let valuesToConcatInDataSourceFilter = Object.assign({}, $scope.filterValues);
		let fields = templateManager.container.getActiveWidget().filterProperties.fields;
		fields.forEach(f => {
			if(f.skipAutoFilter) 
				delete valuesToConcatInDataSourceFilter[f.name];
		});
		
		for (let filterNmae in valuesToConcatInDataSourceFilter) {
			let valueToConcat = valuesToConcatInDataSourceFilter[filterNmae];
			if(valueToConcat === "") {
				valueToConcat = 'T';	
			}
			let filterToChange = widget.dataSourceFilter.find(filter => filter.name === filterNmae);
			if(filterToChange) {
				filterToChange.value = 	valueToConcat;
			} else {
				widget.dataSourceFilter.push({
					name: filterNmae,	
					operator: "=",	
					value: valueToConcat,	
				});
			}
		}
		
	}
	
	$scope.applyFilter = function() {
		let activeWidget = templateManager.container.getActiveWidget();
		if(activeWidget.filterProperties.beforeFilterApply){
			evaluateExpression(activeWidget.filterProperties.beforeFilterApply, 
				Object.assign({}, ApplicationContext), true);
		}
		buildFilter(activeWidget);
		return activeWidget.reload();
	};	
	
	// @Deprecated
	$scope.onValueSelected = function (fieldValue, fieldName) {
		let activeWidget = templateManager.container.getActiveWidget();
		if(!activeWidget.filterValue) {
			activeWidget.filterValue = {};
		}
		activeWidget.filterValue[fieldName] = fieldValue;
	};
	
	function evaluateExpression(expression, context, withoutAtChar) {
		if(withoutAtChar || expression.match(/\@(.*?)\@/g)) {
			expression = expression.replace(/\@/g, '');
			return $scope.$eval(expression, context);
		} else {
			return expression;
		}
	}
	
	let reload = function(){
		this.aditionalOptions = this.aditionalOptions || [];
		ApplicationContext.RestEngine.request(this.dataSource.name, {
			requestType: "FilterData",
			itemsPerPage: 500,
			page: 1,
		    filter: this.dataSourceFilter || []
		}).then(response => {
		        $scope.fieldValues[this.name] = this.aditionalOptions.concat(response.dataset.filial);
		});
	};
	
	$scope.initFields = function(){
		const activeWidget = templateManager.container.getActiveWidget();
		activeWidget.filterValues = $scope.filterValues;
		let fields = activeWidget.filterProperties.fields;
		for (let i = 0; i < fields.length; i++) {
			let defaultValue = fields[i].default;
			if(defaultValue) {
				defaultValue = evaluateExpression(defaultValue);
				$scope.filterValues[fields[i].name] = defaultValue;
			}
			if(fields[i].events) {
				let fieldOnChangeEvent = fields[i].events.find((e) => e.name === 'FieldOnChange');
				if(fieldOnChangeEvent && fieldOnChangeEvent.code) {
					fields[i].onchange = fieldOnChangeEvent.code;
				}
			}
			if(fields[i].type === undefined || fields[i].type === 'select'){
				if(fields[i].dataSource && fields[i].dataSource.rest) {
					fields[i].reload = reload.bind(fields[i]);
					fields[i].reload();
				} else {
					$scope.fieldValues[fields[i].name] = fields[i].dataSource.data;
				}
			} else if(fields[i].type === 'toggle') {
				fields[i].valueToTrue = fields[i].valueToTrue || true; 
				fields[i].valueTofalse = fields[i].valueTofalse || false; 
			}
		}
		
		/**
		 * Usar quando o lazyLoad do dataSource da widget ativa estiver true.
		 * Força a realizar o filtro com um dataSourceFilter modificado de
		 * acordo com o filtro atual carregado.
		 */
		if (activeWidget.applyFilterOnInit) {
			$scope.applyFilter();
		}
	}
	
	$scope.triggerOnChange = function(field) {
		if(field.onchange) {
			let activeWidget = templateManager.container.getActiveWidget();
			evaluateExpression(
				field.onchange, 
				Object.assign(ApplicationContext, {
					filterValues: $scope.filterValues,
					currentWidget: activeWidget
				}), 
				true
			);
		}
	};
	
	$scope.initTest = function(){
		getData();	
	};
	
	$scope.beforeSelectAutoCompleteInitialized = (autoCompleteScope) => {
		/**
		 * the select auto complete field definition is not inside a widget,
		 * so we need to factor a dataSource by our own.
		 * :/
		 */
		autoCompleteScope.field.dataSource = metaDataFactory.dataSourceFactory(
			autoCompleteScope.field.dataSource,
			autoCompleteScope.field.dataSourceFilter,
			autoCompleteScope.field
		);
	};

	$scope.onSelectAutoCompleteInitialized = (autoCompleteScope) => {
		autoCompleteScope.__fieldRow = $scope.filterValues[autoCompleteScope.field.name];
        Object.defineProperty($scope.filterValues, autoCompleteScope.field.name, {
            get: function() {
                return autoCompleteScope.__fieldRow;
            },
            set: function(newValue) {
                autoCompleteScope.__fieldRow = newValue;
				// autoCompleteScope.widget.editRow[autoCompleteScope.field.name] = newValue;
				let selectedElement = autoCompleteScope.field.dataSource.data.find(dataSourceElement => {
					return dataSourceElement[autoCompleteScope.field.name] === autoCompleteScope.__fieldRow;
				});
				if(selectedElement){
					autoCompleteScope.descriptionText = autoCompleteScope.getDescriptionString(selectedElement);
				}
            },
            enumerable: true,
            configurable: true
		});
		// compute initial descrption text
		if(!autoCompleteScope.descriptionText && autoCompleteScope.__fieldRow){
			let selectedElement = autoCompleteScope.field.dataSource.data.find(dataSourceElement => {
				return dataSourceElement[autoCompleteScope.field.name] === autoCompleteScope.__fieldRow;
			});
			if(selectedElement){
				autoCompleteScope.descriptionText = autoCompleteScope.getDescriptionString(selectedElement);
			}
			
		}
	};
}

// FILE: approval_components/js/controllers/FooterNavigationController.js
function FooterNavigationController($scope, $element, ScreenService, ZhUtil, ApplicationContext){
    
    let footerHtmlNode = $element.get(0);
    let footerProperties;
    window.approvalFooter = {};
    
    function renderElements() {
        if(footerProperties) {
            let first = true;
            footerProperties.elements.forEach(elementDetails => {
                let newElement = createElement(elementDetails);
                newElement.name = ScreenService.i18n(elementDetails.name);
                if(first){
                    newElement.classList.add('active');
                    first = false;
                }
                footerHtmlNode.appendChild(newElement);
            });
        }
    }
    
    /**
        called on ng-init
    */
    $scope.onAttach = function(){
        footerProperties = $scope.templateManager.containers.zeedhi_project.footerProperties;
        renderElements();
    };
    
    function createElement(elementDetails) {
        let elementRoot = document.createElement('span');
        elementRoot.classList.add('footer-element');
        elementRoot.textContent = ScreenService.i18n(elementDetails.name);

        let icon = document.createElement('i');
        icon.classList.add('fas');
        icon.classList.add('fa-' + elementDetails.icon);
        elementRoot.appendChild(icon);
        
        bindListeners(elementRoot, elementDetails);
        return elementRoot;
    }
    
      function getApplicationContextWithEventArgs(eventOwner, eventDetails) {
        ApplicationContext.args = {
            owner: eventOwner,
            e: eventDetails,
        };
        return ApplicationContext;
    }
    
    function bindListeners(element, elementDetails) {
        element.addEventListener('click', event => {
            if($scope.$root.activityStack) {
                $scope.$root.activityStack.forEach(stackPopFunction => stackPopFunction());
                $scope.$root.activityStack = [];
            }
            elementDetails.name = ScreenService.i18n(elementDetails.name);
            highlightElement(elementDetails.name);
            if(elementDetails.code) {
                let scopeToEvalute = getApplicationContextWithEventArgs(elementDetails, event);
                $scope.$eval(elementDetails.code,scopeToEvalute);
            } else if(elementDetails.containerName){
                ScreenService.openWindow(elementDetails.containerName);
            }
        });
    }
    
    function highlightElement(elementName) {
        let footerElements = footerHtmlNode.children;
        for (let i = footerElements.length ; i--; ) {
            if(footerElements[i].classList.contains('active') && ScreenService.i18n(footerElements[i].name) !== ScreenService.i18n(elementName)) {
                footerElements[i].classList.remove('active');
            } else if(footerElements[i].name === elementName) {
                footerElements[i].classList.add('active');
            }
        }
    }
    
    window.approvalFooter.highlightElement = highlightElement;
}

// FILE: approval_components/js/controllers/LanguageController.js
'use strict';
function LanguageController($scope, ScreenService, templateManager){
        
    $scope.languageConstructor = function() {
        const language = JSON.parse(localStorage.getItem('language')) || 'pt_br';

        document.getElementsByClassName('flag-selected')[0].classList.add("flag");
        document.getElementsByClassName('flag-selected')[0].classList.remove("flag-selected");

        document.getElementsByName(language)[0].classList.add("flag-selected");
        document.getElementsByName(language)[0].classList.remove("flag");
    };
    
    function removeSelectedFlag() {
        document.getElementsByClassName('flag-selected')[0].classList.add("flag");
        document.getElementsByClassName('flag-selected')[0].classList.remove("flag-selected");
    };

    function selectFlag(e) {
        removeSelectedFlag();
        e.target.classList.add("flag-selected");
        e.target.classList.remove("flag");
    };
    
    $scope.translate = function (e) {
        localStorage.setItem('language', e.target.name);
        selectFlag(e);
        ScreenService.setLanguage(e.target.name).then(function() {
            templateManager.container.getWidget('login').redraw();
        });
    };
}


// FILE: approval_components/js/controllers/QRScannerController.js
"use strict";
function QRScannerController($scope, ApplicationContext, MessageService) {

    $scope.UIManager = ApplicationContext.UIManager;
    // TODO: make function scoped locally
    $scope.widget.__showCameraPreview = this.__showCameraPreview = () => {
        document.body.classList.add('transparent-bg');
        // maybe an array isn'n the best model, but it's o que tem para janta
        $scope.__widgetVisibilities = $scope.templateManager.container.widgets.map(widget => widget.isVisible); 
        ApplicationContext.UIManager.pushState(() => {
            document.body.classList.remove('transparent-bg');
            for (let i = 0; i < $scope.templateManager.container.widgets.length; i++) {
                const widget = $scope.templateManager.container.widgets[i];
                widget.isVisible = $scope.__widgetVisibilities[i];
            }
            delete $scope.__widgetVisibilities;
            'QRScanner' in window && QRScanner.destroy();
            ApplicationContext.UIManager.goBack();
        });
    };

    // TODO: make function scoped locally
    $scope.widget.__hideCameraPreview = this.__hideCameraPreview = () => {
        ApplicationContext.UIManager.goBack();
    };

    // let bindVisibilityAccessFunctions = function bindVisibilityAccessFunctions() {
    //     Object.defineProperty($scope.widget, 'isVisible', {
    //         set: (newValue) => {
    //             $scope.widget._isVisible = this._isVisible = newValue;
    //         },
    //         get: () => $scope.widget._isVisible = this._isVisible,
    //         // enumerable: true,
    //         // configurable: true
    //     });
    // };
    // // constructor(){
    // // $scope.widget..$ = this.$scope = $scope;
    // $scope.widget._isVisible = this._isVisible = $scope.widget._isVisible;
    // bindVisibilityAccessFunctions();
    // }

    $scope.widget.startScan = this.startScan = (autoEnd = true) =>
        new Promise((resolve, reject) => {
            if('QRScanner' in window) {
                $scope.scanRejector = reject;
                QRScanner.show();
                QRScanner.scan((err, text) => {
                    if (err) {
                        reject(err);
                    } else {
                        resolve(text);
                    }
                    if(autoEnd){
                        this.endScan();
                    }
                });
            } else {
                MessageService.getMessageByCode('QRSCANNER_NOT_AVALIA').then(function(message) {
                    reject(message);
                });
            }
        });


    $scope.widget.endScan = this.endScan = () => {
        if('QRScanner' in window) {
            QRScanner.destroy();
        }
    };

    let requestPermission = () =>
        new Promise((resolve, reject) => {
            if(QRScanner) {
                QRScanner.prepare((err, status) => {
                    if (err) {
                        reject(err);
                    } if (status.authorized) {
                        resolve(true);
                    } else if (status.denied) {
                        resolve(false);
                    } else {
                        resolve(false);
                        // needs better support
                        // we didn't get permission, but we didn't get permanently denied. (On
                        // Android, a denial isn't permanent unless the user checks the "Don't
                        // ask again" box.) We can ask again at the next relevant opportunity.
                    }
                });
            } else {
                MessageService.getMessageByCode('QRSCANNER_NOT_AVALIA').then(function(message) {
                    reject(message);
                });
            }
        });
    ;
    
    $scope.onManualInputCancel = () => {
        // $scope.manualInputPopupValue = "";
        $scope.manualInputActive = false;
    };

    $scope.onManualInputConfirm = (typedValue, autoEnd = false) => {
        if(autoEnd){
            this.endScan();
        }
        $scope.manualInputActive = false;
        // if($scope.scanRejector){
        //     $scope.scanRejector('manuallyTyped');
        //     delete $scope.scanRejector;
        // }
        if($scope.widget.onManualQRCodeTyped){
            $scope.$eval($scope.widget.onManualQRCodeTyped, 
                Object.assign(
                    {
                        typedValue: typedValue || ""
                    }, 
                    ApplicationContext
                )
            );
        }
    };
}

// FILE: approval_components/js/controllers/SelectAutocompleteController.js
'use strict';
function SelectAutocompleteController(
    $scope,
    $element,
    ScreenService,
    ZhUtil,
    maskEngine,
    restEngine,
    $q,
    ZhLogger,
    ApplicationContext
    ){

    /**
     * @type {Object}
     */
    var events = {};
    // document.querySelector('.zh-notification-messages').style.display = 'none';
    var activeRequests = [];
    /**
     * @type {Object}
     * a object that will precache the results of each query
     */
    var cacheBase = {};
    /**
     * the spinner element that indicates progress   
     * @type {HTMLElement}
     */
    var spinner = null;
    /**
     * a reference to the current highlited element of the list   
     * @type {HTMLElement}
     */
    var currentHighletedListElement = null;
    /**
     * @type {Object[]} 
     */
    var filteredAditionalOptions = $scope.field.aditionalOptions || [];
    /**
     * @type {Object[]} 
     */
    var filteredListContents = [];
    /** it will be initialized on the initializeData method
     * @type {listContents}
     */
    var listContents = null;
    /**
     * @type {HTMLElement}
     */
    var rootElement;
    /**
     * @type {HTMLElement}
     */
    var popUp;
    /**
     * @type  {HTMLElement}
     */
    var listElement;
    /**
     * @type {HTMLElement}
     */
    var inputElement;
    /**
     * @type {HTMLElement}
     */
    var zhLoader;
    /**
     * @type {HTMLElement}
     */
    var autoCompletePreviewElement;
    /**
     * @type {HTMLElement}
     */
    var labelElement;
    // /**
    //  * @type {HTMLElement}
    //  */
    // var headerLabelElement;
    /**
     * @type {HTMLElement}
     */
    var clearButtonElement;
    /**
     * @type {Object[]}
     */
    var fieldInfo = [];

    /**
     * an element that will be used as a basis to render each list element 
     * @type {HTMLElement}
     */
    var baseListRowToClone;

    var filterConditions = {
        name: "",
        value: "%%",
        operator: "LIKE_ALL"
    };

    var shouldPreventKeyUp = false;
    var lastUpdateTime;
    var updateRequestReference = null;
    var deferredToCancelRequest = false;

    var cacheFirstResultSet = function() {
        cacheBase[""] = listContents;
    };

    /**
     * schedule new http requests and filter the current elements
     * @param {KeyboardEvent} [event] 
     */
    var keyStrokeHandler = function(event) {
        if (event.keyCode > 45 || event.keyCode === 8) {
            //console.log('%ckeyCode: ' + event.keyCode + "   key: '" + event.key + "'", 'color: orange');
            // var begining = performance.now(); // LOG TIME
            if (activeRequests.length) {
                if (activeRequests[0].reqText === inputElement.value) {
                    return;
                }
                else {
                    cancelRequest();
                }
            }
            if (cacheBase[inputElement.value]) {
                filteredListContents = cacheBase[inputElement.value];
                renderListElements(cacheBase[inputElement.value]);
                // //console.log('rendering : ' + (performance.now() - begining)); // LOG TIME
                return;
            }
            filteredAditionalOptions = filterElements(inputElement.value, $scope.field.aditionalOptions || [], true);
            var filtered = filterElements(inputElement.value, null, true);
            renderListElements(filtered);
            // //console.log('rendering : ' + (performance.now() - begining)); // LOG TIME
            // showAutoCompleteText();
            // var timeSinceUpdate = performance.now() - lastUpdateTime;
            // if (timeSinceUpdate >= 200) {
            //     update();
            // }
            // else {
            //     updateRequestReference = setTimeout(update, 200);
            // }
            update();
        }
        shouldPreventKeyUp = true;
    };

    /**
     * handles the command keys keypress, or propagates the event chain for the keyStrokeHandler to resolve
     * @param {KeyboardEvent} [event] 
     */
    var commandKeyStrokeHandler = function(event) {
        var code = event.keyCode;
        if (code > 45 || code === 8) {
            // do nothing and let the keyStrokeHandler handle the keyup event 
            shouldPreventKeyUp = false;
            return;
        }
        else if (code === 40 /* down arrow */ ) {
            if (currentHighletedListElement.parentElement.lastChild !== currentHighletedListElement) {
                currentHighletedListElement.classList.remove('active');
                currentHighletedListElement = currentHighletedListElement.nextElementSibling;
                currentHighletedListElement.classList.add('active');
            }
        }
        else if (code === 38 /* up arrow */ ) {
            if (currentHighletedListElement.parentElement.firstChild !== currentHighletedListElement) {
                currentHighletedListElement.classList.remove('active');
                currentHighletedListElement = currentHighletedListElement.previousElementSibling;
                currentHighletedListElement.classList.add('active');
            }
        }
        else if (code === 13) /* enter */ {
            confirm(null, currentHighletedListElement);
        }
        shouldPreventKeyUp = true;
    };

    /**
     * shows the first element of the filteredListContents array
     */
    var showAutoCompleteText = function() {
        autoCompletePreviewElement.value = getFormatedString(filteredListContents[0], 0);
    };

    var refreshLastUpdateTime = function() {
        lastUpdateTime = performance.now();
    };
    var hideSpinner = function() {
        spinner.classList.add('hidden');
    };
    var onRequestComplete = function() {
        hideSpinner();
        refreshLastUpdateTime();
        //console.log('%cTHE REQUEST WAS SUCESSFULLY COMPLETED', 'color: purple;');
    };
    /**
     * @returns {Promise<null>} 
     */
    var update = function() {
        if (updateRequestReference !== null) {
            clearTimeout(updateRequestReference);
            updateRequestReference = null;
        }
        filterConditions.value = "%" + inputElement.value + "%";
        return fetchData().then(refreshLastUpdateTime);
    };

    var cancelRequest = function() {
        var numberOfCanceledRequests = activeRequests.length;
        ZhLogger.setLevel(999);
        ZhLogger.previousLevel = ZhLogger.level;
        while (activeRequests.length) {
            activeRequests.pop().resolve();
        }
        //console.log('%c' + numberOfCanceledRequests + ' requests were CANCELED ', 'color: white;background-color:red;');
    };

    var initiliazefieldInfo = function() {
        if ($scope.field.fields) {
            for (var i = 0; i < $scope.field.fields.length; i++) { // TODO: use reverse for instead
                var label = ($scope.field.fields[i].label ? ($scope.field.fields[i].label + ": ") : "");
                fieldInfo.push({
                    "label": label,
                    "name": $scope.field.fields[i].name,
                    "mask": $scope.field.fields[i].mask || false
                });
                filterConditions.name += $scope.field.fields[i].name + '|';
            }
            filterConditions.name = filterConditions.name.substr(0, filterConditions.name.length - 1);
        }
        else {
            // var labelWithNoExtraFields = ($scope.field.label ? ($scope.field.label + ": ") : "");
            fieldInfo.push({
                "label": "" /*labelWithNoExtraFields*/ ,
                "name": $scope.field.name,
                "mask": $scope.field.mask || false
            });
            filterConditions.name = $scope.field.name;
        }
        $scope.field.dataSource.itemsPerPage = 10;
        $scope.field.dataSource.disableLoader = true;

        if($scope.field.events){
            let fieldOnChangeEvent = $scope.field.events.find(event => event.name === 'FieldOnChange');
            if(fieldOnChangeEvent){
                events.FieldOnChange = fieldOnChangeEvent.code;
            }
        }
    };

    var initiliazeDataSetter = function() {
        if ($scope.field.dataSource) {
            Object.defineProperty($scope.field.dataSource, 'data', {
                get: function() {
                    return listContents;
                },
                set: function(newValue) {
                    if (newValue === listContents) {
                        return;
                    }
                    newValue = [...filteredAditionalOptions].concat([...newValue]);
                    filteredListContents = listContents = newValue;
                    // theGreatCacheMachine[inputElement.value] = listContents;
                    // //console.log("%cCached '" + inputElement.value + "', with length: " + listContents.length, 'color: purple');
                    renderListElements(listContents);
                    if(!$scope.field.descriptionTextRendered){
                        let selectedElement = $scope.field.dataSource.data.find(dataSourceElement => {
                            return dataSourceElement[$scope.field.name] === $scope.__fieldRow;
                        });
                        if(selectedElement){
                            $scope.descriptionText = $scope.getDescriptionString(selectedElement);
                        }
                    }
                },
                enumerable: true,
                configurable: true
            });
        }

        

    };

    var doCache = function(cacheKey, cacheList) {
        cacheBase[cacheKey] = cacheList;
        //console.log("%cCached '" + cacheKey + "', with length: " + cacheList.length, 'color: purple');
    };

    var initiliazeData = function() {
        if ($scope.field.dataSource.data) {
            filteredListContents = listContents = $scope.field.dataSource.data;
            // cacheFirstResultSet();
            let arrayToRender = listContents;
            if(!filteredAditionalOptions.every(aditionalOption => listContents.includes(aditionalOption))){
                listContents = filteredAditionalOptions.concat(listContents);
            }
            renderListElements(arrayToRender);
        }
        else {
            listContents = [];
        }
        if (!$scope.field.dataSource.dataSourceFilter) {
            $scope.field.dataSource.dataSourceFilter = [];
        }
    };

    $scope.autoCompleteConstructor = function() {
        rootElement = $element.get(0);
        listElement = rootElement.querySelector('#element-list');
        inputElement = rootElement.querySelector('#content');
        // labelElement = rootElement.querySelector('#fieldLabel');
        // switchElement = rootElement.querySelector('#switch');
        autoCompletePreviewElement = rootElement.querySelector('#autocomplete-preview');
        clearButtonElement = rootElement.querySelector('#clear');
        clearButtonElement.addEventListener('click', clear);
        spinner = rootElement.querySelector('.select-autocomplete-spinner');
        popUp = rootElement.querySelector('.select-autocomplete-pop-up');
        inputElement.addEventListener('keyup', keyStrokeHandler);
        inputElement.addEventListener('keydown', commandKeyStrokeHandler);
        zhLoader = document.querySelector('.zh-background-loading');
        let headerLabelElement = rootElement.querySelector('.select-autocomplete-field-label');
        headerLabelElement.textContent = ScreenService.i18n($scope.field.label);
        let backButton = rootElement.querySelector('#back-button');
        backButton.addEventListener('click', $scope.closePopUp);
        inputElement.placeholder = ScreenService.i18n("Pesquise") +  " " + ScreenService.i18n($scope.field.label);
        

        baseListRowToClone = rootElement.querySelector('.select-autocomplete-row').cloneNode(true);
        baseListRowToClone.style.display = 'flex';
        if(!$scope.isMultiple) {
            let checkbox = baseListRowToClone.querySelector('input[type=checkbox]');
            checkbox.style.display = 'none';
        }
        
        // if ('zh-mode' in $scope.field) {
        //     rootElement.classList.add('zh-mode');
        // }

        if($scope.beforeSelectAutoCompleteInitialized) {
            $scope.beforeSelectAutoCompleteInitialized($scope);
        }
        initiliazefieldInfo();
        initiliazeData();
        initiliazeDataSetter();
        if($scope.onSelectAutoCompleteInitialized) {
            $scope.onSelectAutoCompleteInitialized($scope);
        }
    };

    /**
     * @param {Object[]} listElements an array of json objects to be rendered  
     */
    function renderListElements(listElements) {
        //render new list

        while (listElement.lastChild) {
            listElement.removeChild(listElement.lastChild);
        }
        for (var index = 0; index < 9 && index < listElements.length; index++) {
            var newHtmlListElement = baseListRowToClone.cloneNode(true);
            newHtmlListElement.children[1].children[0].textContent = getFormatedString(listElements[index], 0);
            newHtmlListElement.children[1].children[1].children[0].textContent = getFormatedString(listElements[index], 1);
            newHtmlListElement.children[1].children[1].children[1].textContent = getFormatedString(listElements[index], 2);

            newHtmlListElement.realIndex = index;
            newHtmlListElement.addEventListener('click', confirm);
            listElement.appendChild(newHtmlListElement);
        }
        if (index > 0) {
            if (currentHighletedListElement)
                currentHighletedListElement.classList.remove('active');
            currentHighletedListElement = listElement.children[0];
            currentHighletedListElement.classList.add('active');
        }
    }


    /**
     * @param {Object} row - a object to extract the values for the display string
     * @returns {string} a string with the descriptionFields that are in the row concated with pipe
     */
    $scope.getDescriptionString = function(row){
        if($scope.field.descriptionFields){
            return $scope.field.descriptionFields.map(descriptionField => row[descriptionField])
                .join(' | ')
        } else {
            return row[$scope.field.name];
        }
    }

    /**
     * returns an String with the field name concatenated, or an empty string if the object doesn't have that field  
     * @returns {String} 
     * @param {Obect} object 
     * @param {Number} index the index of the field that is being searched on the fieldInfo array 
     */
    function getFormatedString(object, index) {
        if (fieldInfo.length <= index && object) {
            return "";
        }
        else if (object.hasOwnProperty(fieldInfo[index].name)) {
            var value = object[fieldInfo[index].name];
            if (fieldInfo[index].mask) {
                value = Util.applyMask(value, fieldInfo[index].mask, maskEngine, ZhUtil);
            }
            return fieldInfo[index].label + value;
        }
        else {
            return "";
        }
    }

    /**
     * @param {string} stringToSearch 
     * @param {object[]} arraryToSearch
     * @param {boolean} [cleanReturn] - if is set to true the method doesn't affect 'filteredListContents' and returns a new array instead 
     */
    "use strict";
    function filterElements(stringToSearch, arraryToSearch, cleanReturn) {
        var workArray;
        arraryToSearch = arraryToSearch || listContents;
        if (!cleanReturn) {
            workArray = filteredListContents;
        }
        workArray = [];
        for (var i = 0; /*i < 9 && */ i < arraryToSearch.length; i++) {
            if (stringIsInJSON(arraryToSearch[i], stringToSearch))
                workArray.push(arraryToSearch[i]);
        }
        if (cleanReturn)
            return workArray;
    }

    /**
     * //TODO refactor docs// search all properties of a json object to find a string
     * @param {Object} object 
     * @param {String} stringToMatch 
     */
    function stringIsInJSON(object, stringToMatch) {
        stringToMatch = stringToMatch.toLowerCase();
        for (var i = fieldInfo.length; i--;) {
            if (('' + object[fieldInfo[i].name]).toLowerCase().indexOf(stringToMatch) !== -1)
                return true;
        }
        return false;
    }

    /** 
     * @param {MouseEvent} event - a event to extract the list html element
     * @param {HTMLElement} listElement - a list element to select
     */
    var confirm = function(event, listElement) {
        var element;
        var shouldForceDigest = true;
        if (event) {
            element = event.currentTarget;
        }
        else {
            element = listElement;
            shouldForceDigest = true;
        }
        var newValueObject = filteredListContents[element.realIndex];
        // labelElement.classList.add('rui-txt-input-label-val');
        setValue(newValueObject);
        
        inputElement.blur();
        clearButtonElement.style.display = 'initial';
        $scope.closePopUp(); 
        // ApplicationContext.UIManager.goBack();
        if (shouldForceDigest) {
            $scope.$apply();
        }
    };

    /**
     * refreshes the ui, changes internal variables and trigger listeners
     * TODO: refactor this doc
     * @param {Object} newValue 
     */
    let setValue = function(newValue){
        let newValueString = newValue[$scope.field.name];
        $scope.__fieldRow = newValueString; // TODO refactor this line
        inputElement.value = newValueString;
        $scope.descriptionText = $scope.getDescriptionString(newValue);
        
        if($scope.onSelectAutoCompletedValueChanged){
            $scope.onSelectAutoCompletedValueChanged(newValue, $scope.field);
        }

        if(events.FieldOnChange) {
            $scope.$eval(events.FieldOnChange, Object.assign({}, ApplicationContext, {
                args: {
                    owner: {field: $scope.field},
                    row: $scope.field.widget.currentRow
                }
            }));
        }
        insertOutData(newValue);
    }

    /**
     * clear the value of the current row, and the applied filter
     * @param {MouseEvent} event 
     * @param {Boolean} [preventUpdate] wheter or not the clear function should reload the data 
     */
    function clear(event, preventUpdate) {
        if (activeRequests)
            cancelRequest();
        $scope.__fieldRow = null;
        // labelElement.classList.remove('rui-txt-input-label-val');
        inputElement.value = "";
        clearButtonElement.style.display = 'none';
        // if (!preventUpdate) {
        //     update();
        // }
        if (!cacheBase.hasOwnProperty('')) {
            update();
        }
        else {
            $scope.field.dataSource.data = cacheBase[""];
        }
        destroyOutData();
    }

    /**
     * 
     * @param {Object} rowObject -- a ZhRowObject to extract the data and insert on the currentRow
     */
    function insertOutData(rowObject) {
        for (var key in $scope.field.outData) {
            var value = rowObject[key];
            $scope.field.widget.currentRow[$scope.field.outData[key]] = value;
        }
    }

    function destroyOutData() {
        for (var key in $scope.field.outData) {
            delete $scope.field.widget.currentRow[$scope.field.outData[key]];
        }
    }

    async function fetchData() {
        //console.log('%cFetch ' + filterConditions.value, 'color: blue;');
        var deferredToCancelRequest = $q.defer();
        deferredToCancelRequest.reqText = inputElement.value;
        activeRequests.push(deferredToCancelRequest);
        spinner.classList.remove('hidden');
        // preventZhLoaderFromShowing();
        ScreenService.hideLoader();
        var dataSourceFilter = $scope.field.dataSourceFilter || 
                               $scope.field.dataSource.dataSourceFilter || [];
        var newFilter = dataSourceFilter.slice(0); // make a copy of the zhDataSourceFilter to keep it pristine
        newFilter.push({
            name: filterConditions.name,
            value: "%" + inputElement.value + "%",
            operator: "LIKE_ALL"
        }); // append our custom conditions into the new array
        var promisseToReturn = await $scope.field.dataSource.load(
            newFilter,
            null,
            null,
            () => {},
            () => {}, -1,
            deferredToCancelRequest.promise,
            () => {}
        ).then(function(resultSet) {
            if (resultSet) {
                doCache(deferredToCancelRequest.reqText, 
                    filteredAditionalOptions.concat(resultSet.dataset[Object.keys(resultSet.dataset)[0]])
                );
            }
            else {
                // console.log('canceled');
                ZhLogger.setLevel(ZhLogger.previousLevel);
            }
            deferredToCancelRequest = false;
        }, function(errorArg) {}
        ).finally(function() {
            hideSpinner();
        });
        // ScreenService.hideLoader();
        return promisseToReturn;
    }

    $scope.openPopUp = () => {
        popUp.classList.add('visible');
        // $scope.hideHeaderActions();
        inputElement.focus();
        // ApplicationContext.UIManager.pushState(() => {
        //     $scope.closePopUp(); 
        //     // $scope.restoreHeaderActions();
        // });
        zhLoader.classList.add('invisible');
    };
    
    $scope.closePopUp = () => {
        popUp.classList.remove('visible');
        zhLoader.classList.remove('invisible');
    }


    let hideSelectAutoCompleteAction = {
        "name": "cancelEdit",
        "label": "cancel",
        "ruiIcon": "fa-arrow-left",
        "isVisible": true,
        "showAsAction": "back",
        "events": [{
            "name": "ActionEvent",
            "code": "UIManager.goBack();"
        }]
    };

    $scope.hideHeaderActions = () => {
        $scope.headerStateBackup = {
            headerActions: $scope.widget.headerActions,
            showMenu: $scope.widget.showMenu,
            sortable: $scope.widget.sortable,
            showFilter: $scope.widget.showFilter,
            label: $scope.widget.container.label
        };
        $scope.widget.headerActions = [angular.copy(hideSelectAutoCompleteAction)];
        $scope.widget.showMenu = false;
        $scope.widget.sortable = false;
        $scope.widget.showFilter = false;
        $scope.widget.container.label = $scope.field.label;
    };

    $scope.restoreHeaderActions = () => {
        $scope.widget.container.label = $scope.headerStateBackup.label;
        delete $scope.headerStateBackup.label;

        for (const backupKeyName in $scope.headerStateBackup) {
            $scope.widget[backupKeyName] = $scope.headerStateBackup[backupKeyName];
            delete $scope.headerStateBackup[backupKeyName];
        }
    };
}


// FILE: approval_components/js/controllers/TimelineController.js
function TimelineController($scope) {

    $scope.statusToIcon = {
        "error": "fa-times",
        "exclamation": "fa-exclamation",
        "checked": "fa-check",
        "block": "fa-lock"
    };

    function getOffset(el) {
        el = el.getBoundingClientRect();
        return {
            left: el.left + window.scrollX,
            top: el.top + window.scrollY
        }
    }

    $scope.calculate = function () {
            let histList = document.getElementsByClassName('comment-card');
            for (let i = 0; i < histList.length; i++) {

                let curitemList = histList[i];
                let itemI = curitemList.children[0];
                let iTop = getOffset(itemI).top;

                let line = curitemList.children[curitemList.children.length - 1];

                if (histList[i + 1]) {
                    let nextItemList = histList[i + 1];
                    let nextitemI = nextItemList.children[0];
                    let nextiTop = getOffset(nextitemI).top;
                    let heightOfLine = nextiTop - iTop;

                    line.style.height = (heightOfLine+41) + 'px';
                }

            }
    };
}

// FILE: approval_components/js/directives/ApprovalMask.js
//Configuration to ApprovalDirectives be in aplication.
var ApprovalDirectives = angular.module("ApprovalDirectives", []);
ZeedhiApp.requires.push('ApprovalDirectives');
// console.log(ZeedhiApp)
ApprovalDirectives.directive('appMask', function() {
    
    function maskApplyOnValueField($scope) {
        let field = $scope.field;
        switch(field.mask) {
            case 'currency':
                $scope.data[field.name] = formatMoney($scope.data[field.name]);
            break;
            case 'productAmount':
                $scope.data[field.name] = formatProductAmount($scope.data[field.name]);
            break;
            case 'productCode':
                $scope.data[field.name] = formatProductCode($scope.data[field.name]);
            break;
        }
    }
    
    function formatMoney(value) {
        value = parseFloat(value);
        return "R$" + value.toFixed(2).replace('.', ',').replace(/(\d)(?=(\d{3})+\,)/g, "$1.");
    }
    
    function formatProductAmount(value) {
        value = parseFloat(value);
        value = value.toFixed(3);
        return value.replace('.', ',');
    }
    
    function formatProductCode(value) {
        var mask = getLocalVar('CDPICTPROD');
        var pos = findLetterPositions(mask, '.');

        pos.forEach(f => {
            value = [value.slice(0, f), '.', value.slice(f)].join('');
        });
        
        return value;
    }

    function findLetterPositions(str, letter) {
        let positions = [];
        for (let i = 0; i < str.length; i++) {
            if (str[i] === letter) {
                positions.push(i);
            }
        }
        return positions;
    }
    
    return {
        link: maskApplyOnValueField
    };
    
});
