var LoginService = function(eventAggregator, EngineService, templateManager, ZHPromise, restEngine, MessageService, requestDataSourceEngine, localStorageEngine) {

    // ------------------------ Public Functions -------------------------

    var self = this;
    this.expiredSessionMethod = null;

    this.containsSericeUrls = function() {
        if (window.serviceUrls && window.serviceUrls.length > 0) {
            return true;
        } else {
            return false;
        }
    };

    this.getBaseUri = function() {
        return EngineService.getLocalVar('baseUri');
    };

    this.getProtocol = function() {
        return EngineService.getLocalVar('protocol');
    };

    this.login = function(data, attempts) {
        var filter;

        try {
            filter = [{
                name: "EMAIL",
                operator: "=",
                value: data.email
            }, {
                name: "PASSWORD",
                operator: "=",
                value: data.password
            }, {
                name: 'PRODUCT_ID',
                operator: '=',
                value: window.projectId
            }, {
                name: 'REQUESTER_URL',
                operator: '=',
                value: document.location.href
            }, {
                name: 'ATTEMPTS',
                operator: '=',
                value: attempts
            }];
        } catch (e) {
            var defer = ZHPromise.defer();
            defer.reject(e);
            return defer;
        }

        return EngineService.authenticate(filter);
    };

    this.logout = function() {
        return EngineService.logout().then(function() {
            EngineService.setLocalVar('login', 'notLogged');
            localStorage.removeItem('authToken');
            openWindow(templateManager.project.mainWindow);
        });
    };

    this.testUrl = function(baseUri) {
        try {
            alias = EngineService.getProductAlias();
        } catch (e) {
            var defer = ZHPromise.defer();
            defer.reject(e);
            return defer;
        }

        if (baseUri.indexOf(alias) === -1) {
            baseUri = baseUri + alias;
        }

        return EngineService.testUrl(baseUri).then(function() {
            EngineService.buildUrls();
            return baseUri;
        }, function() {
            return 'Address not found.';
        });

    };

    this.reLogin = () => {
        return new Promise((resolve, reject) => {
            let token = getLocalVar('authToken');
            let isLogged = (getLocalVar('login') === 'logged');
            if (token && isLogged) {
                return restEngine.post('sup#/getLoginByToken', {
                    "requestType": "Row",
                    "row": {
                        token: token
                    }
                }).then(userData => {
                    let dataToLogin = {
                        email: userData.dataset.user.EMAIL,
                        password: userData.dataset.user.PS,
                    };
                    this.login(dataToLogin, 1).then(response => {
                        templateManager.container.getActiveWidget().reload();
                        resolve();
                    }, error => {
                        reject(error);
                    });
                });
            } else {
                MessageService.getMessageByCode('NO_PROV_TOKEN').then(function(message) {
                    reject(message);
                });
            }
        });
    };

    eventAggregator.subscribe("EventOnRequestStart", function(eventData) {
        var validateExpiredSessionRoute = 'validateExpiredSessionApprove';
        var logoutRoute = 'lib_logout';
        var loginRoute = 'login';
        var validateUserRoute = 'validateUser';

        var requestConfig = eventData.requestConfig;
        if (requestConfig && requestConfig.url && shouldAddRequestInfo(requestConfig.url) && templateManager.container) {
            requestConfig.data.origin = requestConfig.data.origin || {};
            var widget;
            var container = templateManager.loadingContainerJson;
            if (container) {
                widget = getFirstWidgetWithDataSourceRest(container.widgets);
                if (!widget) {
                    widget = container.widgets[0];
                }
            } else {
                container = templateManager.container;
                widget = container.getActiveWidget();
            }

            requestConfig.data.origin.containerLabel = container.label || container.name;
            requestConfig.data.origin.widgetLabel = widget.label || widget.name;
        }

        var route = requestConfig.url.split('/').pop();
        if (route != validateExpiredSessionRoute) {
            factoryExpiredSessionMethods();
        }
        if (route != logoutRoute && route != loginRoute) {
            var token = localStorage.getItem('LOGIN_TOKEN');
            var hash = localStorage.getItem('LOGIN_HASH');
            var keepConnected = localStorage.getItem('LOGIN_KEEP_CONNECTED');
            if (window.externalTokens) {
                window.externalTokens.some(function(externalToken) {
                    if (eventData.requestConfig.url.includes(externalToken.url)) {
                        token = externalToken.token;
                        keepConnected = externalToken.keepConnected || keepConnected;
                        return true;
                    }
                    return false;
                });
            }
            if (token) {
                requestConfig.headers['OAuth-Token'] = token;
            }
            if (hash) {
                requestConfig.headers['OAuth-Hash'] = hash;
                requestConfig.headers['OAuth-Project'] = window.tekLoginProjectId;
            }
            if (route == validateUserRoute) {
                keepConnected = 'Yes';
            }
            requestConfig.headers['OAuth-KeepConnected'] = keepConnected;
        }
    });

    var shouldAddRequestInfo = function(url) {
        return !url.match(/.json/i) &&
            !url.match(/.html/i) &&
            !url.match(/.svg/i);
    };

    var getFirstWidgetWithDataSourceRest = function(widgets) {
        for (var i = 0; i < widgets.length; i++) {
            if (widgets[i].dataSource && widgets[i].dataSource.rest == true) {
                return widgets[i];
            } else {
                return getFirstWidgetWithDataSourceRest(widgets[i]);
            }
        }
    };

    var factoryExpiredSessionMethods = function() {
        if (templateManager.project && templateManager.project.onRequestIdleTimeout) {
            if (self.expiredSessionMethod) {
                clearTimeout(self.expiredSessionMethod);
            }
            self.expiredSessionMethod = Util.doAsync(validateExpiredSession, templateManager.project.onRequestIdleTimeout);
        }
    };

    var validateExpiredSession = function() {
        var timeout = templateManager.project.onRequestIdleTimeout;
        var filter = [{
            name: 'TIMEOUT',
            value: timeout,
            operator: '='
        }];

        var userdata = getLocalVar('USERDATA');

        if (userdata && userdata.LOGGED) {
            requestDataSourceEngine.getDataSource('sup#/validateExpiredSessionApprove', filter).then(function(response) {
                if (response && response.dataset && response.dataset.expiredSessionInfo) {
                    var expiredSessionInfo = response.dataset.expiredSessionInfo;
                    if (expiredSessionInfo.IS_EXPIRED) {
                        eventAggregator.publish('LibExpiredSession');
                    } else {
                        if (self.expiredSessionMethod) {
                            clearTimeout(self.expiredSessionMethod);
                        }
                        Util.doAsync(validateExpiredSession, expiredSessionInfo.TIMEOUT);
                    }
                }
            });
        }
    };

    function logoutExpiredSession() {
        return EngineService.logout().then(function() {
            EngineService.setLocalVar('login', 'notLogged');
            localStorage.removeItem('authToken');
            openWindow(templateManager.project.mainWindow);

            var userData = getLocalVar('USERDATA');

            var newUserData = {
                'USER_ID': userData.USER_ID,
                'USER_NAME': userData.USER_NAME,
                'USER_EMAIL': userData.USER_EMAIL,
                'TOKEN': null,
                'LOGGED': false,
            };

            localStorageEngine.setLocalVar('USERDATA', newUserData);
        });
    }

    function expiredLogout() {
        if ('device' in window) { // if isn't running in device do nothing
            let deviceDataToDelete = {
                UUID: device.uuid,
                NRAPLICATIVO: 5
            };
            restEngine.post('sup#/crud/device/remove', {
                "requestType": "Row",
                "row": deviceDataToDelete
            }).then(logoutExpiredSession());
        } else {
            logoutExpiredSession();
        }
    }

    eventAggregator.subscribe("LibExpiredSession", function() {
        if (!templateManager.container.getActiveWidget().tekLibExpiredSession) {
            expiredLogout();
        }
    });
};

Configuration(function(ContextRegister) {
    ContextRegister.register('LoginService', LoginService);
});
